import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

/**
 * Created by gaetan on 17.03.16.
 */
public class MainClient implements SendTurnRedRunnable.OnTurnRedMessage, ReceiveReactionButtonRunnable.OnReactionButtonReceived, TCPClient.OnStartExperiment {

    private int mNumberRedSent = 0;
    private Pair humanReactionTime = null;
    private TCPClient mTcpClient = null;
    private SendTurnRedRunnable mSendTurnRedRunnable;
    private ReceiveReactionButtonRunnable mReceiveReactionButtonRunnable;

    private int mNumberButtonPressed = 0;
    private String mParticipantID = "";
    private boolean mIsSimulation;
    private boolean mArgos;

    private String mGlobalLog = "";

    private Thread mSendTurnRedThread;
    private Thread mReceiveReactionThread;

    private ArrayList<Pair> mReactionTimeList = new ArrayList<Pair>();


    public MainClient(String participantID, boolean isSimulation, boolean argos) {
        mParticipantID = participantID;
        mIsSimulation = isSimulation;
        mArgos = argos;

        mTcpClient = new TCPClient(this);
        mTcpClient.run();
    }

    @Override
    public void onTurnRedMessageSent(String robot) {
        long nanoTime = System.nanoTime();

        humanReactionTime = new Pair();
        humanReactionTime.setX(nanoTime);

        mReactionTimeList.add(humanReactionTime);

        System.out.println("Send turn red to: " + robot);
        mGlobalLog += "Send turn red to: " + robot + "\n";

        mNumberRedSent++;
    }

    @Override
    public void onTurnRedMessageEnd() {
        mTcpClient.sendMessage("stop".getBytes());
        mReceiveReactionButtonRunnable.Stop();

        System.out.println("******* END **********");
        System.out.println("Number red sent: " + mNumberRedSent);
        System.out.println("Number of button pressed: " + mNumberButtonPressed);

        mGlobalLog += "******* END **********\n";
        mGlobalLog += "Number red sent: " + mNumberRedSent + "\n";
        mGlobalLog += "Number of button pressed: " + mNumberButtonPressed + "\n";

        for (int i=0; i<mReactionTimeList.size(); ++i) {
            //System.out.println(mReactionTimeList.get(i).toString());
            System.out.println(mReactionTimeList.get(i).getDifference()/1000000000.0);
            mGlobalLog += mReactionTimeList.get(i).getDifference()/1000000000.0 + "\n";
        }

        SaveResultToFile();

    }

    private void SaveResultToFile() {
        PrintWriter writer = null;
        PrintWriter globalLogWriter = null;
        try {
            if (mIsSimulation) {
                if (mArgos) {
                    writer = new PrintWriter("data/"+mParticipantID+"_ar.txt", "UTF-8");
                    globalLogWriter = new PrintWriter("data/"+mParticipantID+"_log_ar.txt", "UTF-8");
                } else {
                    writer = new PrintWriter("data/"+mParticipantID+"_vr.txt", "UTF-8");
                    globalLogWriter = new PrintWriter("data/"+mParticipantID+"_log_vr.txt", "UTF-8");
                }
            } else {
                writer = new PrintWriter("data/"+mParticipantID+"_rr.txt", "UTF-8");
                globalLogWriter = new PrintWriter("data/"+mParticipantID+"_log_rr.txt", "UTF-8");
            }

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        writer.println(mNumberRedSent);
        writer.println(mNumberButtonPressed);


        for (int i=0; i<mReactionTimeList.size(); ++i) {
            if (mReactionTimeList.get(i).getY() != 0)
                writer.println(mReactionTimeList.get(i).getDifference()/1000000000.0 + "\t" + mReactionTimeList.get(i).getX() + "\t" + mReactionTimeList.get(i).getY());
            else
                writer.println("NaN" + "\t" + mReactionTimeList.get(i).getX() + "\t" + mReactionTimeList.get(i).getY());

        }

        globalLogWriter.println(mGlobalLog);
        globalLogWriter.close();
        writer.close();
    }


    @Override
    public void onReactionButtonReceived() {
        long nanoTime = System.nanoTime();
        if (mReactionTimeList.size() > 0) {
            Pair p = mReactionTimeList.get(mReactionTimeList.size() - 1);
            // avoid case where user presses button while no robot turned red
            if (p.getY() == 0) {
                p.setY(nanoTime);
            }
        }
        mNumberButtonPressed++;
        System.out.println("User pressed button");
        mGlobalLog += "User pressed button\n";
    }

    @Override
    public void onStartExperiment() {
        mSendTurnRedRunnable = new SendTurnRedRunnable(mTcpClient, this, mIsSimulation);
        mReceiveReactionButtonRunnable = new ReceiveReactionButtonRunnable(this);

        mSendTurnRedThread = new Thread(mSendTurnRedRunnable);
        mSendTurnRedThread.start();

        mReceiveReactionThread = new Thread(mReceiveReactionButtonRunnable);
        mReceiveReactionThread.start();
    }


    private class Pair {
        private long x;
        private long y;

        public Pair() {
            x = 0;
            y = 0;
        }

        public Pair(long x_, long y_) {
            x = x_;
            y = y_;
        }

        public void setX(long x_) {
            x = x_;
        }

        public void setY(long y_) {
            y = y_;
        }

        public long getX() {
            return x;
        }

        public long getY() {
            return y;
        }

        public long getDifference() {
            return (y - x);
        }

        @Override
        public String toString() {
            return "x: " + String.valueOf(x) + "\n" +
                    "y: " + String.valueOf(y) + "\n" +
                    "y-x: " + String.valueOf(y - x);

        }
    }
}
