import java.util.ArrayList;
import java.util.Random;

/**
 * Created by gaetan on 28.01.15.
 */
public class SendTurnRedRunnable implements Runnable{
    private static Random mRandom;    // pseudo-mRandom number generator
    private static long mSeed;        // pseudo-mRandom number generator mSeed
    private long mTicks = 0;
    private long mTickBeforeNextRed = 0;
    private int mLastIdx = 255;
    private ArrayList<String> robotList = new ArrayList<String>(20);

    protected OnTurnRedMessage mOnTurnRedMessageListener = null;

    private boolean mIsSimulation;

    protected volatile Boolean mKeepRunning;
    protected TCPClient mTcpClient;

    public SendTurnRedRunnable(TCPClient tcpClient, MainClient listener, boolean isSimulation) {
        mTcpClient = tcpClient;
        mSeed = System.currentTimeMillis();
        mRandom = new Random(mSeed);
        mOnTurnRedMessageListener = listener;
        mIsSimulation = isSimulation;
        initRobotList();

    }

    public SendTurnRedRunnable(TCPClient tcpClient, long seed, MainClient listener) {
        initRobotList();
        mRandom = new Random(seed);
        mTcpClient = tcpClient;
        mOnTurnRedMessageListener = listener;

    }

    private void initRobotList() {
        String robotId = "";
        if (mIsSimulation) {
            for (int i = 0; i < 20; ++i) {
                robotId = "epuck" + Integer.toString(i);
                robotList.add(robotId);
            }
        } else {
            //FIXME add manually real robot ids here
            robotList.add("epuck33");
            robotList.add("epuck45");
            robotList.add("epuck54");
            robotList.add("epuck37");
            robotList.add("epuck50");
            robotList.add("epuck40");
            robotList.add("epuck58");
            robotList.add("epuck46");
            robotList.add("epuck44");
            robotList.add("epuck39");
            robotList.add("epuck51");
            robotList.add("epuck29");
            robotList.add("epuck49");
            robotList.add("epuck47");
            //robotList.add("epuck35");
            robotList.add("epuck41");
            robotList.add("epuck55");
            robotList.add("epuck34");
            robotList.add("epuck36");
            robotList.add("epuck48");

        }
    }

    public void run() {
            mKeepRunning = true;
            while (mKeepRunning) {
                try {
                    Thread.sleep(100);
                    mTicks += 1;
                    if (mTickBeforeNextRed == 0) {
                        sendRobotTurnRed();
                    } else {
                        --mTickBeforeNextRed;
                    }
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                if (mTicks > 600) {
                    mKeepRunning = false;
                    mOnTurnRedMessageListener.onTurnRedMessageEnd();
                }
            }

    }

        private void sendRobotTurnRed() throws InterruptedException {
            if (bernoulli(0.02)) {
                int idx = mRandom.nextInt(robotList.size());
                if (idx == mLastIdx) {
                    if (mLastIdx > 0) {
                        idx = mLastIdx - 1;
                    } else {
                        idx = mLastIdx + 1;
                    }
                    System.out.println("changed random chosen");
                }
                mLastIdx = idx;
                mOnTurnRedMessageListener.onTurnRedMessageSent(robotList.get(idx));
                mTickBeforeNextRed = 20;

                mTcpClient.sendMessage(robotList.get(idx).getBytes());
                //Thread.sleep(1000);
            }
        }

        public static boolean bernoulli(double p) {
            if (!(p >= 0.0 && p <= 1.0))
                throw new IllegalArgumentException("Probability must be between 0.0 and 1.0");
            return uniform() < p;
        }

        public static double uniform() {
            return mRandom.nextDouble();
        }

    public interface OnTurnRedMessage {
        public void onTurnRedMessageSent(String robot);
        public void onTurnRedMessageEnd();

    }
}
