/**
 * Created by gaetan on 22.10.15.
 */

import java.io.*;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.Scanner;


public class TCPClient  {


    public static final String ANSI_RED = "\u001B[31m";

    private static int SERVER_PORT = 4321;
    private static final int MAX_MESSAGE_SIZE = 12000;
    private static final String SERVER_IP = "localhost";

    private volatile Boolean mRun;
    private Socket mSocket = null;
    private DataOutputStream mOutput = null;
    private DataInputStream mInput = null;
    private byte[] mBufferMsg = new byte[MAX_MESSAGE_SIZE];
    private byte[] mReceivedMessageSize = new byte[4];
    private boolean hasSentExperimentStart = false;

    private OnStartExperiment mOnStartExperimentListener = null;
    MainClient mMainClientReference = null;

    private boolean mIsConnected = false;

    public TCPClient(MainClient listener) {
        mMainClientReference = listener;
        mOnStartExperimentListener = listener;
    }

    public void run() {
        mRun = true;

        try {
            // Connect socket to the server, wait 5 sec before timeout
            InetSocketAddress inetSocketAddress = new InetSocketAddress(SERVER_IP, SERVER_PORT);
            mSocket = new Socket();
            mSocket.connect(inetSocketAddress, 5000);

            mOutput = new DataOutputStream(mSocket.getOutputStream());
            mInput = new DataInputStream(mSocket.getInputStream());

            mIsConnected = true;
            while (mRun) {
                // FIXME: will never receive (read) any message !!
                //readIncomingMessages();
                //readUserInput();
               if (!hasSentExperimentStart) {
                   readUserInput();
                   mOnStartExperimentListener.onStartExperiment();

               }
                else {
                   readIncomingMessages();
                }
            }
            closeConnection();
        } catch (SocketTimeoutException e) {
            System.out.println("Can't connect to server, it seems the server is down");
        } catch (IOException e) {
            System.out.println("Connection to server failed: " + e.toString());
        }
    }


    public void Stop() {
        mRun = false;
    }
    public void closeConnection() {
        try {
            mOutput.close();
            mSocket.close();
            mIsConnected = false;
        } catch (IOException e) {
            System.out.println("An error occured while closing output and socket: " + e.toString());
        }
    }

    private void readUserInput() {
        Scanner reader = new Scanner(System.in);
        System.out.println("Press 'a' to start ARGoS experiment, 'v' to start the 'virtual reality' experiment and 'r' to start real robots: ");

        String s = reader.next();
        if (s.equals("a") || s.equals("v") || s.equals("r")) {
            sendMessage(s.getBytes());
            hasSentExperimentStart = true;
        }
        return;
    }

    private void readIncomingMessages() {
        int bytesRead;
        try {
            bytesRead = mInput.read(mReceivedMessageSize, 0, 4);

            int size = ((mReceivedMessageSize[0] & 0xFF) << 24) | ((mReceivedMessageSize[1] & 0xFF) << 16) |
                    ((mReceivedMessageSize[2] & 0xFF) << 8) | (mReceivedMessageSize[3] & 0xFF);

            if (bytesRead == 4 && size > 0 && size <= MAX_MESSAGE_SIZE)  {
                int total_bytes_read = 0;
                while (total_bytes_read < size) {
                    bytesRead = mInput.read(mBufferMsg, total_bytes_read, size - total_bytes_read);
                    total_bytes_read += bytesRead;
                }
                if (total_bytes_read == size) {
                    String sMessageReceived = filterMessage(new String(mBufferMsg, 0, size));

                    //System.out.println("Message received from server: " + sMessageReceived);
                }
                else {
                    System.out.println("Error reading data from server -- disconnecting");

                    mRun = false;
                    mIsConnected = false;
                }
            }
            else if (bytesRead == -1) {
                System.out.println("Byte read = -1, closing connection with socket");
                mRun = false;
                mIsConnected = false;
            }
        } catch (IOException e) {
            System.out.println("In IOocception, closing connection with socket");
            mRun = false;
            mIsConnected = false;
        }
    }

    public void sendMessage(byte[] bufferToSend) {
        if (bufferToSend.length > MAX_MESSAGE_SIZE) {
            System.out.println("SendMessage: message probably too long: " + bufferToSend.length +"(max is: "+MAX_MESSAGE_SIZE+")");
            return;
        }
        try {
            long size = (long)bufferToSend.length;
            mOutput.writeLong(size);
            mOutput.write(bufferToSend);
            mOutput.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String filterMessage(String message) {
        message = message.replaceAll("\\x00", "");

        return message;
    }

    public interface OnStartExperiment {
        public void onStartExperiment();
    }


}